from bidict import bidict

nsmap = bidict(
    {
        "gen": "http://www.opengis.net/citygml/generics/2.0",
        "gml": "http://www.opengis.net/gml",
        "core": "http://www.opengis.net/citygml/2.0",
        "bldg": "http://www.opengis.net/citygml/building/2.0",
        "tran": "http://www.opengis.net/citygml/transportation/2.0",
        "brid": "http://www.opengis.net/citygml/bridge/2.0",
        "xlink": "http://www.w3.org/1999/xlink",
        "xAL": "urn:oasis:names:tc:ciq:xsdschema:xAL:2.0",
        "uro": "https://www.geospatial.jp/iur/uro/2.0",
        "dem": "http://www.opengis.net/citygml/relief/2.0",
        "frn": "http://www.opengis.net/citygml/cityfurniture/2.0",
        "urf": "https://www.geospatial.jp/iur/urf/2.0",
        "luse": "http://www.opengis.net/citygml/landuse/2.0",
        "veg": "http://www.opengis.net/citygml/vegetation/2.0",
    }
)

prefectures = [
    "北海道",
    "青森県",
    "岩手県",
    "宮城県",
    "秋田県",
    "山形県",
    "福島県",
    "茨城県",
    "栃木県",
    "群馬県",
    "埼玉県",
    "千葉県",
    "東京都",
    "神奈川県",
    "新潟県",
    "富山県",
    "石川県",
    "福井県",
    "山梨県",
    "長野県",
    "岐阜県",
    "静岡県",
    "愛知県",
    "三重県",
    "滋賀県",
    "京都府",
    "大阪府",
    "兵庫県",
    "奈良県",
    "和歌山県",
    "鳥取県",
    "島根県",
    "岡山県",
    "広島県",
    "山口県",
    "徳島県",
    "香川県",
    "愛媛県",
    "高知県",
    "福岡県",
    "佐賀県",
    "長崎県",
    "熊本県",
    "大分県",
    "宮崎県",
    "鹿児島県",
    "沖縄県",
]
